#include <cstdio>
#include <cstdlib>
#include <algorithm>
#include <vector>
using namespace std;

#define assert(f) { if(!(f)) { fprintf(stderr,"Assertion failed: "); fprintf(stderr,#f); fprintf(stderr,"\n"); exit(1); } }

typedef unsigned int UI;
typedef long long LL;
typedef pair<LL,int> PLI;
typedef pair<int,LL> PIL;

int rand15() { return rand()%(1<<15); }
int rand30() { return rand15()*(1<<15)+rand15(); }
int rand(int L,int R)
{
	assert(L<=R);
	return rand30()%(R-L+1)+L;
}
LL RAND(){return LL(rand30())*(1<<30)+rand30();}
LL RAND(LL L,LL R)
{
	assert(L<=R);
	return RAND()%(R-L+1)+L;
}

int rndA = 168726318;
int rndB = 768978978;

const int maxT = 320000;
const int maxL = 36;
const int maxN = (maxL+1)/2;
const LL maxV = 1000000000000000000LL;

const int base7 = 1;
const int base5 = base7*(maxN+1);
const int base3 = base5*(maxN+1);
const int base2 = base3*(2*maxN+1);
const int base0 = base2*(3*maxN+1);
int digit_incr[10]={
	0,
	0,
	base2,
	base3,
	2*base2,
	base5,
	base2+base3,
	base7,
	3*base2,
	2*base3
};

UI dp[maxN+1][base0];
UI res[maxL+1][base0];
UI res0[maxL+1];
LL V_by_i7[base0];

const int PRIME = 999983;
PLI table[PRIME];

void precalc()
{
	UI pw9[maxL+1]={1},pw10[maxL+1]={1};
	for(int n=1;n<=maxL;n++)
	{
		pw9[n]=9*pw9[n-1];
		pw10[n]=10*pw10[n-1];
		int n1=(n-1)/2;
		res0[n] = (pw10[n1]-pw9[n1])*pw9[n-n1];
	}

	LL pw2[3*maxN+1]={1};
	for(int n=1;n<=3*maxN;n++) pw2[n]=2*pw2[n-1];
	LL pw3[2*maxN+1]={1};
	for(int n=1;n<=2*maxN;n++) pw3[n]=3*pw3[n-1];
	LL pw5[maxN+1]={1};
	for(int n=1;n<=maxN;n++) pw5[n]=5*pw5[n-1];
	LL pw7[maxN+1]={1};
	for(int n=1;n<=maxN;n++) pw7[n]=7*pw7[n-1];

	dp[0][0]=1;
	for(int n=0;n<=maxN;n++)
	{
		for(int n2=0,i2=0; n2<=3*n;n2++,i2+=base2)
		for(int n3=0,i3=i2;n3<=2*n;n3++,i3+=base3)
		for(int n5=0,i5=i3;n5<=n;n5++,i5+=base5)
		for(int n7=0,i7=i5;n5+n7<=n;n7++,i7++)
		{
			UI cnt=dp[n][i7];
			if(!cnt) continue;
			LL V=pw2[n2]*pw3[n3]*pw5[n5]*pw7[n7];
			if(n==maxN)
			{
				assert(table[V%PRIME].first==0);
				table[V%PRIME]=PLI(V,i7);
				V_by_i7[i7]=V;
			}
			if(n<maxN)
			{
				for(int d=1;d<=9;d++)
					dp[n+1][i7 + digit_incr[d]] += cnt;
			}
			if(n>0)
			{
				for(int m2=0,j2=0; m2<=n2;m2++,j2+=base2)
				for(int m3=0,j3=j2;m3<=n3;m3++,j3+=base3)
				for(int m5=0,j5=j3;m5<=n5;m5++,j5+=base5)
				for(int m7=0,j7=j5;m7<=n7;m7++,j7++)
				{
					res[2*n][i7-j7] += cnt * dp[n][j7];
					res[2*n-1][i7-j7] += cnt * dp[n-1][j7];
				}
			}
		}
	}
}

UI solve(int L,LL V)
{
	if(V==0) return res0[L];
	PLI cur=table[V%PRIME];
	if(cur.first!=V) return 0;;
	return res[L][cur.second];
}

typedef pair<int,LL> PIL;

void write(int ftst,vector<PIL> test)
{
	fprintf(stderr,"ftst=%d\n",ftst);
	char name[11];
	sprintf(name,"%d.in",ftst);
	freopen(name,"wb",stdout);
	sprintf(name,"%d.out",ftst);
	FILE* out=fopen(name,"wb");
	int T=test.size();
	assert(1<=T && T<=maxT);
	printf("%d\n",T);
	random_shuffle(test.begin(), test.end());
	for(int t=0;t<T;t++)
	{
		int L=test[t].first;
		assert(1<=L && L<=maxL);
		LL V=test[t].second;
		assert(0<=V && V<maxV);
		printf("%d %lld\n",L,V);
		fprintf(out,"%u\n",solve(L,V));
	}
}

// will contain all pairs (L, V) with non-zero answer
vector<PIL> good;

void gen0(int ftst)
{
	srand(rndA * ftst + rndB);
	vector<PIL> test;
	for(int L=1;L<=maxL;L++)
		test.push_back(PIL(L,0));
	for(int i=0;i<good.size();i++)
		test.push_back(good[i]);
	write(ftst,test);
}

void gen1(int ftst)
{
	srand(rndA * ftst + rndB);
	vector<PIL> test;
	for(int t=0;t<maxT;t++)
		test.push_back(PIL(rand(maxL-1,maxL),0));
	write(ftst,test);
}

void gen2(int ftst)
{
	srand(rndA * ftst + rndB);
	vector<PIL> test;
	for(int t=0;t<maxT;t++)
		test.push_back(PIL(rand(1,maxL), RAND()%maxV));
	write(ftst,test);
}

void gen3(int ftst)
{
	srand(rndA * ftst + rndB);
	vector<PIL> test;
	LL V=1;
	for(int i=0;i<maxN;i++) V*=9;
	for(int t=0;t<maxT;t++)
		test.push_back(PIL(rand(maxL-1,maxL), V));
	write(ftst,test);
}

vector<LL> all2357;
int p2357[4]={2,3,5,7};
void rec(int i,LL V)
{
	if(i==4) {
		all2357.push_back(V);
		return;
	}
	int p=p2357[i];
	for(;;)
	{
		rec(i+1,V);
		if(V>(maxV-1)/p) break;
		V*=p;
	}
}

void gen4(int ftst)
{
	srand(rndA * ftst + rndB);
	vector<LL> a=all2357;
	sort(a.begin(), a.end());
	reverse(a.begin(), a.end());
	fprintf(stderr,"cnt=%d\n",a.size());
	vector<PIL> test;
	// number of values V for which we will have 5 values of L
	int n5 = maxT - 4 * a.size();
	for(int i=0;i<a.size() && test.size() < maxT;i++)
	{
		LL V=a[i];
		int mark[maxL]={0};
		if(test.size() < maxT)
			test.push_back(make_pair(maxL,V));
		if(test.size() < maxT)
			test.push_back(make_pair(maxL-1,V));
		int maxk = i < n5 ? 3 : 2;
		for(int k=0;k<maxk;k++)
			for(;;)
			{
				int L=rand(1,maxL-2);
				if(mark[L]) continue;
				mark[L]=1;
				if(test.size() < maxT)
					test.push_back(make_pair(L,V));
				break;
			}
	}
	write(ftst, test);
}

void gen5(int ftst)
{
	vector<LL> a;
	for(int i=0;i<all2357.size();i++)
	{
		LL V=all2357[i];
		if(V<1000000000) a.push_back(V);
	}
	vector<PIL> test;
	for(int t=0;t<maxT;t++)
		test.push_back(PIL(rand(maxL-1,maxL),a[rand()%a.size()]));
	write(ftst,test);
}

void gen6(int ftst)
{
	vector<LL> a;
	for(int i=0;i<all2357.size();i++)
	{
		LL V=all2357[i];
		if(V<1000) a.push_back(V);
	}
	vector<PIL> test;
	for(int t=0;t<maxT;t++)
		test.push_back(PIL(rand(maxL-1,maxL),a[rand()%a.size()]));
	write(ftst,test);
}

void gen7(int ftst)
{
	srand(rndA * ftst + rndB);
	vector<PIL> test;
	for(int t=0;t<maxT;t++)
		test.push_back(PIL(rand(maxL-1,maxL), t));
	write(ftst,test);
}

const int maxP = 4600000;
char sieve[maxP];
void gen8(int ftst)
{
	srand(rndA * ftst + rndB);
	vector<PIL> test;
	for(int p=2;p<maxP && test.size()<maxT;p++) if(!sieve[p])
	{
		int id=rand30()%good.size();
		int L=good[id].first;
		LL V=good[id].second;
		LL dif = RAND(1,1000000000) * p;
		if(V>dif) V-=dif; else V+=dif;
		test.push_back(PIL(L,V));
		for(int n=p;n<maxP;n+=p) sieve[n]=1;
	}
	write(ftst,test);
}

void gen9(int ftst)
{
	srand(rndA * ftst + rndB);
	vector<PIL> a;
	for(int L=1;L<=maxL;L++)
		a.push_back(PIL(L,0));
	for(int i=0;i<good.size();i++)
		a.push_back(good[i]);
	vector<PIL> test;
	for(int t=0;t<maxT;t++)
	{
		int id=rand30()%a.size();
		int L=a[id].first;
		LL V=a[id].second;
		V+=rand()%2?-1:1;
		if(V<0) V=1;
		if(V>=maxV) V--;
		test.push_back(PIL(L,V));
	}
	write(ftst,test);
}

void gen10(int ftst)
{
	srand(rndA * ftst + rndB);
	vector<PIL> test;
	LL V=7;
	for(int i=0;i<maxN;i++) V*=8;
	for(int t=0;t<maxT;t++)
		test.push_back(PIL(rand(maxL-1,maxL), V));
	write(ftst,test);
}

void (*gen[])(int)={gen0,gen1,gen2,gen3,gen4,gen5,gen6,gen7,gen8,gen9,gen10};

int main()
{
	precalc();

	// finding of all good pairs (L, V)
	for(int L=1;L<=maxL;L++)
		for(int i7=0;i7<base0;i7++)
			if(res[L][i7]) good.push_back(PIL(L,V_by_i7[i7]));

	// needed for several tests
	// all2357 will contain all numbers of the form 2^n2 * 3^n3 * 5^n5 * 7^n7 < 10^18
	all2357.clear();
	rec(0,1);

	for(int ftst=0;ftst<=10;ftst++)
		gen[ftst](ftst);
	return 0;
}
